/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// SWIG input file of the 'dart' module

%feature("autodoc","1");

%module(docstring=
"'dartw' module: full potential solver 2018-2021
(c) ULg - A&M",
directors="0",
threads="1"
) dartw
%{

#include "dart.h"

#include "fwkw.h"
#include "tboxw.h"
#include "dartw.h"

%}


%include "fwkw.swg"

// ----------- MODULES UTILISES ------------
%import "tboxw.i"

// ----------- FLOW CLASSES ----------------
%include "dart.h"

%shared_ptr(dart::Fluid);
%shared_ptr(dart::Assign);
%shared_ptr(dart::Initial);
%shared_ptr(dart::Dirichlet);
%shared_ptr(dart::Freestream);
%shared_ptr(dart::Wake);
%shared_ptr(dart::Kutta);
%shared_ptr(dart::Blowing);
%shared_ptr(dart::Body);
%shared_ptr(dart::Problem);
%shared_ptr(dart::Solver);
%shared_ptr(dart::Picard);
%shared_ptr(dart::Newton);
%shared_ptr(dart::Adjoint);

%include "wViscSolver.h"
%include "wBLRegion.h"
%include "wTimeSolver.h"
%include "wViscFluxes.h"
%include "wClosures.h"

%include "wFluid.h"
%include "wAssign.h"
%include "wFreestream.h"
%warnfilter(509); // Shadowed constructor. Code compiling and running.
%include "wWake.h"
%include "wKutta.h"
%immutable dart::Body::Cl; // read-only variable (no setter)
%immutable dart::Body::Cd;
%immutable dart::Body::Cs;
%immutable dart::Body::Cm;
%immutable dart::Body::nLoads;
%include "wBody.h"
%include "wBlowing.h"
%warnfilter(+509);

%immutable dart::Problem::msh; // read-only variable (no setter)
%immutable dart::Problem::nDim;
%immutable dart::Problem::alpha;
%immutable dart::Problem::beta;
%immutable dart::Problem::M_inf;
%immutable dart::Problem::S_ref;
%immutable dart::Problem::c_ref;
%immutable dart::Problem::x_ref;
%include "wProblem.h"

%immutable dart::Solver::pbl; // read-only variable (no setter)
%immutable dart::Solver::linsol;
%immutable dart::Solver::relTol;
%immutable dart::Solver::absTol;
%immutable dart::Solver::maxIt;
%immutable dart::Solver::nthreads;
%immutable dart::Solver::verbose;
%immutable dart::Solver::nIt;
%immutable dart::Solver::phi;
%immutable dart::Solver::rPhi;
%immutable dart::Solver::vPhi;
%immutable dart::Solver::U;
%immutable dart::Solver::rho;
%immutable dart::Solver::M;
%immutable dart::Solver::Cp;
%immutable dart::Solver::Cl;
%immutable dart::Solver::Cd;
%immutable dart::Solver::Cs;
%immutable dart::Solver::Cm;
%include "wSolver.h"
%include "wPicard.h"
%include "wNewton.h"

%immutable dart::Adjoint::sol; // read-only variable (no setter)
%immutable dart::Adjoint::def;
%immutable dart::Adjoint::linsol;
%include "wAdjoint.h"
