#include "wBLRegion.h"
#include "wClosures.h"
#include <iostream>
#include <vector>
#include <cmath>

using namespace tbox;
using namespace dart;


BLRegion::BLRegion()
{
  std::cout << "Coucou de BLRegion\n" << std::endl;

  closSolver = new Closures();

  
} // end BLRegion


BLRegion::~BLRegion()
{ 
  delete closSolver;
  std::cout << "~BLRegion()\n";
} // end ~BLRegion

void BLRegion::SetMesh(std::vector<double> locM, std::vector<double> globM)
{
  LocMarkers=locM;
  GlobMarkers=globM;
  nMarkers = locM.size();

  /* Resize all variables */

  U.resize(nVar * nMarkers);
  Ret.resize(nMarkers);
  Cd.resize(nMarkers);
  Cf.resize(nMarkers);
  Hstar.resize(nMarkers);
  Hstar2.resize(nMarkers);
  Hk.resize(nMarkers);
  Cteq.resize(nMarkers);
  Us.resize(nMarkers);
  DeltaStar.resize(nMarkers);
  Delta.resize(nMarkers);
  Me.resize(nMarkers);
  Rhoe.resize(nMarkers);
  VtExt.resize(nMarkers);
  DeltaStarExt.resize(nMarkers);
  XxExt.resize(nMarkers);
  ReExt.resize(nMarkers);
  Regime.resize(nMarkers);
} // End SetMesh

void BLRegion::SetExtVariables(std::vector<double> _xxExt, std::vector<double> _deltaStarExt)
{
  XxExt = _xxExt;
  DeltaStarExt = _deltaStarExt;
}

void BLRegion::SetInvBC(std::vector<double> _Ue,
                        std::vector<double> _Me,
                        std::vector<double> _Rhoe)
{
  VtExt = _Ue;
  Me = _Me;
  Rhoe = _Rhoe;
}// End SetInvBC

void BLRegion::SetStagBC(double Re)
{
  double dv0 = (VtExt[1] - VtExt[0])/(LocMarkers[1] - LocMarkers[0]);

  if (dv0 > 0){
    U[0] = sqrt(0.075/(Re*dv0)); // Theta
  } // End if
  else{
    U[0] = 1e-6; // Theta
  } // end else
  U[1] = 2.23; // H
  U[2] = 0; // N
  U[3] = VtExt[0]; // ue
  U[4] = 0; // Ctau
  
  Ret[0] = U[3]*U[0]*Re;

  if (Ret[0] < 1){
    Ret[0] = 0;
    U[3] = Ret[0]/(U[0]*Re);
  } // End if

  DeltaStar[0] = U[0] * U[1];

  std::vector<double> ClosParam = closSolver->LaminarClosures(U[0], U[1], Ret[0], Me[0], name);

  Hstar[0] = ClosParam[0];
  Hstar2[0] = ClosParam[1];
  Hk[0] = ClosParam[2];
  Cf[0] = ClosParam[3];
  Cd[0] = ClosParam[4];
  Delta[0] = ClosParam[5];
  Cteq[0] = ClosParam[6];
  Us[0] = ClosParam[7];
} // End SetStagBC

void BLRegion::SetWakeBC(double Re, std::vector<double> UpperCond, std::vector<double> LowerCond)
{

  U[0]  = UpperCond[0]+LowerCond[0];                                       /* (Theta_up+Theta_low). */
  U[1]  = (UpperCond[0]*UpperCond[1]+LowerCond[0]*LowerCond[1])/U[0];      /* ((Theta*H)_up+(Theta*H)_low)/(Theta_up+Theta_low). */
  U[2]  = 9;
  U[3] = VtExt[0];                                                         /* Inviscid velocity. */
  U[4] = (UpperCond[0]*UpperCond[4]+LowerCond[0]*LowerCond[4])/U[0];       /* ((Ct*Theta)_up+(Theta)_low)/(Ct*Theta_up+Theta_low). */

  Ret[0] = U[3]*U[0]*Re;                                                   /* Reynolds number based on the momentum thickness. */
  
  if (Ret[0] < 1){
      Ret[0] = 1;
      U[3] = Ret[0] / (U[0]*Re);
  }
  DeltaStar[0] = U[0]*U[1];                                                /* deltaStar */
  
  /* Laminar closures. */

  std::vector<double> ClosParam = closSolver->LaminarClosures(U[0], U[1], Ret[0], Me[0], name);

  Hstar[0] = ClosParam[0];
  Hstar2[0] = ClosParam[1];
  Hk[0] = ClosParam[2];
  Cf[0] = ClosParam[3];
  Cd[0] = ClosParam[4];
  Delta[0] = ClosParam[5];
  Cteq[0] = ClosParam[6];
  Us[0] = ClosParam[7];

  for (size_t k=0; k<nMarkers; ++k){
    Regime[k] = 1;
  }
}

unsigned int BLRegion::GetnVar(){return nVar;}
size_t BLRegion::GetnMarkers(){return nMarkers;}
double BLRegion::GetnCrit(){return nCrit;}
double BLRegion::GetLocMarkers(size_t iPoint){return LocMarkers[iPoint];}
double BLRegion::GetGlobMarkers(size_t iPoint){return GlobMarkers[iPoint];}
double BLRegion::GetVtExt(size_t iPoint){return VtExt[iPoint];}
double BLRegion::GetMe(size_t iPoint){return Me[iPoint];}
double BLRegion::GetRhoe(size_t iPoint){return Rhoe[iPoint];}
double BLRegion::GetXxExt(size_t iPoint){return XxExt[iPoint];}
double BLRegion::GetDeltaStarExt(size_t iPoint){return DeltaStarExt[iPoint];}

void BLRegion::PrintSolution(size_t iPoint)
{
  std::cout << "Point " << iPoint << std::scientific;
  std::cout << "Theta " << U[iPoint * nVar + 0] << " \n" << std::scientific;
  std::cout << "H " << U[iPoint * nVar + 1] << " \n" << std::scientific;
  std::cout << "N " << U[iPoint * nVar + 2] << " \n" << std::scientific;
  std::cout << "ue " << U[iPoint * nVar + 3] << " \n" << std::scientific;
  std::cout << "Ct " << U[iPoint * nVar + 4] << " \n" << std::scientific;
  std::cout << "Regime " << Regime[iPoint] << " \n" << std::scientific;
  }
