#ifndef WBLREGION_H
#define WBLREGION_H

#include "dart.h"
#include "wClosures.h"
#include <vector>
#include <string>

namespace dart
{

class DART_API BLRegion
{

private:

  unsigned int nVar = 5;

  /* Mesh parameters */

  size_t nMarkers;
  std::vector<double> GlobMarkers;            /* Position of the mesh elements in the global frame of reference */
  std::vector<double> LocMarkers;             /* Position of the mesh elements in the local frame of reference  */

  /* External flow variables */

  std::vector<double> Me;
  std::vector<double> Rhoe;
  std::vector<double> VtExt;
  std::vector<double> DeltaStarExt;
  std::vector<double> XxExt;
  std::vector<double> ReExt;

  /* Transition related variables */

  double nCrit=9.0;

public:

  Closures *closSolver;

  std::string name;

   /* Boundary layer variables */

  std::vector<double> U;                      /* Solution vector (theta, H, N, ue, Ct)                          */
  std::vector<double> Ret;                    /* Reynolds number based on the momentum thickness (theta)        */
  std::vector<double> Cd;                     /* Local dissipation coefficient                                  */
  std::vector<double> Cf;                     /* Local friction coefficient                                     */
  std::vector<double> Hstar;
  std::vector<double> Hstar2;
  std::vector<double> Hk;
  std::vector<double> Cteq;
  std::vector<double> Us;
  std::vector<double> Delta;

  std::vector<double> DeltaStar;

  /* Transition related variables */

  std::vector<int> Regime;

  unsigned int transMarker;
  double xtrF;
  double xtr;

  BLRegion();
  ~BLRegion();
  void SetMesh(std::vector<double> locM, std::vector<double> globM);
  void SetInvBC(std::vector<double> _Ue,
                std::vector<double> _Me,
                std::vector<double> _Rhoe);
  void SetExtVariables(std::vector<double> _xxExt, std::vector<double> _deltaStarExt);

  void SetStagBC(double Re);
  void SetWakeBC(double Re, std::vector<double> UpperCond, std::vector<double> LowerCond);

  unsigned int GetnVar();
  size_t GetnMarkers();
  double GetnCrit();
  double GetLocMarkers(size_t iPoint);
  double GetGlobMarkers(size_t iPoint);
  double GetVtExt(size_t iPoint);
  double GetMe(size_t iPoint);
  double GetRhoe(size_t iPoint);
  double GetXxExt(size_t iPoint);
  double GetDeltaStarExt(size_t iPoint);

  void PrintSolution(size_t iPoint);
};
} // namespace dart
#endif //WBLREGION_H