/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WF1EL_H
#define WF1EL_H

#include "dart.h"
#include "wObserver.h"
#include <vector>
#include <Eigen/Dense>

namespace dart
{

/**
 * @brief Vector function to be integrated over an element
 */
class DART_API F1El : public fwk::Observer
{
public:
    F1El() : Observer() {}
    virtual ~F1El() {}

    virtual Eigen::Vector3d eval(tbox::Element const &e, std::vector<double> const &u,
                                 size_t k) const = 0;
    virtual Eigen::Vector3d evalGrad(tbox::Element const &e, std::vector<double> const &u,
                                     size_t k) const = 0;
};

/**
 * @brief Freestream velocity
 */
class DART_API F1ElVi : public F1El
{
    int nDim;           ///< number of dimensions
    double beta;        ///< angle of sideslip
    Eigen::Vector3d v;  ///< freestream velocity vector
    Eigen::Vector3d dv; ///< freestream velocity vector gradient (wrt alpha)

public:
    F1ElVi(int _nDim, double alpha, double _beta = 0);
    virtual void update(double alpha) override;

    virtual Eigen::Vector3d eval(tbox::Element const &e, std::vector<double> const &u,
                                 size_t k) const override;
    virtual Eigen::Vector3d evalGrad(tbox::Element const &e, std::vector<double> const &u,
                                     size_t k) const override;
};

} // namespace dart

#endif //WF1EL_H
